package de.jottyfan.quickiemod.block;

import java.util.HashMap;
import java.util.Map;

import com.mojang.serialization.MapCodec;

import de.jottyfan.quickiemod.blockentity.DrillBlockEntity;
import de.jottyfan.quickiemod.item.ModItems;
import net.minecraft.block.AbstractBlock;
import net.minecraft.block.Block;
import net.minecraft.block.BlockEntityProvider;
import net.minecraft.block.BlockState;
import net.minecraft.block.FallingBlock;
import net.minecraft.block.entity.BlockEntity;
import net.minecraft.block.entity.BlockEntityTicker;
import net.minecraft.block.entity.BlockEntityType;
import net.minecraft.entity.ItemEntity;
import net.minecraft.entity.player.PlayerEntity;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.item.Items;
import net.minecraft.registry.RegistryKey;
import net.minecraft.registry.RegistryKeys;
import net.minecraft.state.StateManager.Builder;
import net.minecraft.state.property.EnumProperty;
import net.minecraft.state.property.IntProperty;
import net.minecraft.text.Text;
import net.minecraft.util.ActionResult;
import net.minecraft.util.Identifier;
import net.minecraft.util.hit.BlockHitResult;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.Direction;
import net.minecraft.world.World;

/**
 *
 * @author jotty
 *
 */
public class BlockDrill extends FallingBlock implements BlockEntityProvider {
	private static final Integer MAX_FUEL = 255;
	public static final IntProperty FUEL = IntProperty.of("fuel", 0, MAX_FUEL);
	public static final EnumProperty<Direction> DIRECTION = EnumProperty.of("direction", Direction.class, Direction.values());

	public BlockDrill(Identifier identifier, Direction direction) {
		super(AbstractBlock.Settings.create().hardness(2.5f).registryKey(RegistryKey.of(RegistryKeys.BLOCK, identifier)));
		setDefaultState(getDefaultState().with(FUEL, 0).with(DIRECTION, direction));
	}

	@Override
	public BlockEntity createBlockEntity(BlockPos pos, BlockState blockState) {
		return new DrillBlockEntity(pos, blockState);
	}

	@Override
	public <T extends BlockEntity> BlockEntityTicker<T> getTicker(World world, BlockState state,
			BlockEntityType<T> type) {
		return (world1, pos, state1, be) -> DrillBlockEntity.tick(world1, pos, state1, be);
	}

	@Override
	protected MapCodec<? extends FallingBlock> getCodec() {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BlockState onBreak(World world, BlockPos pos, BlockState state, PlayerEntity player) {
		Integer fuelLeft = state.get(FUEL);
		world.spawnEntity(
				new ItemEntity(world, pos.getX(), pos.getY(), pos.getZ(), new ItemStack(ModItems.ITEM_CANOLABOTTLE, fuelLeft)));
		return super.onBreak(world, pos, state, player);
	}

	@Override
	protected void appendProperties(Builder<Block, BlockState> builder) {
		builder.add(FUEL).add(DIRECTION);
	}

	@Override
	protected ActionResult onUse(BlockState state, World world, BlockPos pos, PlayerEntity player, BlockHitResult hit) {
		Map<Item, Integer> loadings = new HashMap<>();
		loadings.put(ModItems.ITEM_CANOLABOTTLE, 8);
		loadings.put(ModItems.ITEM_CANOLABOTTLESTACK, 72);
		ItemStack stack = player.getStackInHand(player.getActiveHand());
		Item item = stack.getItem();
		if (stack.isEmpty() || !loadings.containsKey(item)) {
			if (world.isClient()) {
				player.sendMessage(Text.translatable("msg.drill.fuelinfo", state.get(FUEL)), false);
			}
		} else {
			Integer fuelWeight = loadings.get(item);
			if (fuelWeight != null) {
				Integer load = MAX_FUEL - state.get(FUEL);
				if (load < fuelWeight) {
					fuelWeight = load;
				}
				world.setBlockState(pos, state.with(FUEL, state.get(FUEL) + fuelWeight));
				if (item.equals(ModItems.ITEM_CANOLABOTTLE)) {
					world.spawnEntity(
							new ItemEntity(world, pos.getX(), pos.getY(), pos.getZ(), new ItemStack(Items.GLASS_BOTTLE, 1)));
				} else if (item.equals(ModItems.ITEM_CANOLABOTTLESTACK)) {
					world.spawnEntity(
							new ItemEntity(world, pos.getX(), pos.getY(), pos.getZ(), new ItemStack(Items.GLASS_BOTTLE, 9)));
				}
				stack.decrement(1);
			}
		}
		return ActionResult.PASS;
	}
}
