package de.jottyfan.quickiemod.block;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;

import de.jottyfan.quickiemod.item.ModItems;
import net.minecraft.block.AbstractBlock;
import net.minecraft.block.Block;
import net.minecraft.block.BlockState;
import net.minecraft.block.Blocks;
import net.minecraft.entity.ItemEntity;
import net.minecraft.entity.LivingEntity;
import net.minecraft.entity.player.PlayerEntity;
import net.minecraft.item.ItemStack;
import net.minecraft.item.Items;
import net.minecraft.loot.context.LootWorldContext.Builder;
import net.minecraft.registry.RegistryKey;
import net.minecraft.registry.RegistryKeys;
import net.minecraft.server.world.ServerWorld;
import net.minecraft.util.ActionResult;
import net.minecraft.util.Hand;
import net.minecraft.util.Identifier;
import net.minecraft.util.hit.BlockHitResult;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;
import net.minecraft.world.tick.OrderedTick;

/**
 *
 * @author jotty
 *
 */
public class BlockLavahoarder extends Block {

	public BlockLavahoarder(Identifier identifier) {
		super(AbstractBlock.Settings.create().hardness(2.5f).luminance(state -> 15)
				.registryKey(RegistryKey.of(RegistryKeys.BLOCK, identifier)));
	}

	@Override
	public List<ItemStack> getDroppedStacks(BlockState state, Builder builder) {
		List<ItemStack> list = new ArrayList<>();
		list.add(new ItemStack(ModBlocks.BLOCK_LAVAHOARDER));
		return list;
	}

	public static final void spawnRandomItems(World world, BlockPos pos, Integer count) {
		Integer which = new Random().nextInt(10);
		if (which < 1) {
			world.spawnEntity(new ItemEntity(world, pos.getX(), pos.getY(), pos.getZ(),
					new ItemStack(Items.DIAMOND, new Random().nextInt(count + 2))));
		} else if (which < 2) {
			world.spawnEntity(new ItemEntity(world, pos.getX(), pos.getY(), pos.getZ(),
					new ItemStack(Items.EMERALD, new Random().nextInt(count + 1))));
		} else if (which < 3) {
			world.spawnEntity(new ItemEntity(world, pos.getX(), pos.getY(), pos.getZ(),
					new ItemStack(Items.RAW_GOLD, new Random().nextInt(count))));
		} else if (which < 4) {
			world.spawnEntity(new ItemEntity(world, pos.getX(), pos.getY(), pos.getZ(),
					new ItemStack(Items.RAW_IRON, new Random().nextInt(count + 1))));
		} else if (which < 5) {
			world.spawnEntity(new ItemEntity(world, pos.getX(), pos.getY(), pos.getZ(),
					new ItemStack(Items.RAW_COPPER, new Random().nextInt(count + 2))));
		} else if (which < 6) {
			world.spawnEntity(new ItemEntity(world, pos.getX(), pos.getY(), pos.getZ(), new ItemStack(Items.OBSIDIAN)));
		} else if (which < 7) {
			world.spawnEntity(new ItemEntity(world, pos.getX(), pos.getY(), pos.getZ(), new ItemStack(Items.LAPIS_LAZULI)));
		} else if (which < 8) {
			world.spawnEntity(new ItemEntity(world, pos.getX(), pos.getY(), pos.getZ(), new ItemStack(ModItems.ITEM_SULFOR)));
		}
	}

	/**
	 * sucks the lava that touches the block
	 *
	 * @param world the world
	 * @param pos   the pos
	 * @return true if lava was found
	 */
	public static final boolean suckLava(World world, BlockPos pos) {
		if (world == null) {
			return false;
		} else if (Blocks.LAVA.equals(world.getBlockState(pos).getBlock())) {
			world.setBlockState(pos, Blocks.AIR.getDefaultState());
			BlockPos up = pos.up();
			Random random = new Random();
			if (random.nextFloat() > 0.9f) {
				spawnRandomItems(world, up, 2);
			}
			return true;
		}
		return false;
	}

	@Override
	protected void scheduledTick(BlockState state, ServerWorld world, BlockPos pos,
			net.minecraft.util.math.random.Random random) {
		suckLava(world, pos.north());
		suckLava(world, pos.south());
		suckLava(world, pos.east());
		suckLava(world, pos.west());
		suckLava(world, pos.up());
		suckLava(world, pos.down());
		world.getBlockTickScheduler().scheduleTick(OrderedTick.create(this, pos));
	}

	@Override
	protected ActionResult onUse(BlockState state, World world, BlockPos pos, PlayerEntity player, BlockHitResult hit) {
		if (!world.isClient) {
			Hand hand = player.getActiveHand();
			ItemStack handStack = player.getStackInHand(hand);
			if (handStack != null && Items.BUCKET.equals(handStack.getItem())) {
				Integer amount = handStack.getCount();
				ItemStack lavaBucketStack = new ItemStack(Items.LAVA_BUCKET, 1);
				ItemStack emptyBucketStack = new ItemStack(Items.BUCKET, amount - 1);
				if (emptyBucketStack.getCount() < 1) {
					player.setStackInHand(hand, lavaBucketStack);
				} else {
					player.setStackInHand(hand, emptyBucketStack);
					world.spawnEntity(new ItemEntity(world, pos.getX(), pos.getY(), pos.getZ(), lavaBucketStack));
				}
				spawnRandomItems(world, pos, 2);
				world.setBlockState(pos, ModBlocks.BLOCK_EMPTYLAVAHOARDER.getDefaultState());
				world.getBlockTickScheduler().scheduleTick(OrderedTick.create(ModBlocks.BLOCK_EMPTYLAVAHOARDER, pos));
			}
		}
		return ActionResult.SUCCESS; // forbid to empty the just filled lava bucket
	}

	@Override
	public void onPlaced(World world, BlockPos pos, BlockState state, LivingEntity placer, ItemStack itemStack) {
		world.getBlockTickScheduler().scheduleTick(OrderedTick.create(this, pos));
	}
}
